package cn.com.ty.chat.neety.process;

import cn.com.ty.chat.neety.protocol.IMMessage;
import cn.com.ty.chat.neety.support.SensitivewordFilter;
import cn.com.ty.chat.neety.support.WebSocketSupport;
import cn.com.ty.chat.neety.task.RoomInfo;
import cn.com.ty.chat.neety.task.UserInfo;
import com.alibaba.fastjson.JSONObject;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 实时聊天处理类
 *
 * @author sKing
 * @Date 2020年6月12日
 */
@Slf4j
public class ChatProcessor extends WebSocketSupport implements ProcessorInterface {
    /**
     * websoket消息处理
     *
     * @param client
     * @param request
     */
    @Override
    public void process(Channel client, IMMessage request) {
        //获取发送人信息
        String currentUserId = client.attr(USERID).get();

        //获取消息
        String sessionId = request.getSessionid();
        //敏感词过滤
        if (request.getType() == 1){
            request.setMsg(SensitivewordFilter.replaceSensitiveWord(request.getMsg(), 1, "**"));
        }


        //redis只存当天的消息，RedisTask中synRedisChatToDB定时任务会在每天凌晨把消息同步到数据库
        //每次发消息，把消息累加到redis中，推送全量数据给客户端
        redis.lRightPush("ALLCHATS:" + sessionId, JSONObject.toJSONString(request));

        //更新聊天室中的最后一条记录及时间，在app中获取聊天室排序有用
        Optional.ofNullable(redis.get("ROOM:" + sessionId)).ifPresent(s -> {
            RoomInfo roomInfo = JSONObject.parseObject(s, RoomInfo.class);
            roomInfo.setLastChat(request.getMsg());
            roomInfo.setLastTime(request.getTime());

            roomInfo.getUserList().forEach(userInfo -> {
                if (!userInfo.getUserId().equals(currentUserId)){
                    userInfo.setIsRed(1);
                }
            });
            redis.set("ROOM:" + sessionId, JSONObject.toJSONString(roomInfo));
            //通过redis获取房间信息
            //用户正常登录的时候链接就会调用login协议，login会加载聊天室与用户关系到数据
            //当用户已经登录了，附近的人聊天的时候，接收人是不知道自己已经有一个新的会话。
            //出诊问诊时，专家没有接单的情况下是不能进入聊天室的。
            //这种情况会造成，附近的人聊天，接收人不进入聊天室接收不到消息，
            //通过标记类型，附近的人聊天，直接用userchannel变量中找到链接发送给接收人，并把聊天记录存入redis中
            List<Channel> sendChannelList = new ArrayList<>();
            for (int i = 0; roomInfo.getUserList() != null
                    && !roomInfo.getUserList().isEmpty()
                    && i < roomInfo.getUserList().size(); i++) {
                UserInfo userInfo = roomInfo.getUserList().get(i);
                List<Channel> channelList = userChannel.get(userInfo.getUserId());
                if (channelList != null){
                    sendChannelList.addAll(channelList);
                }
            }

            for (Channel channel : sendChannelList) {
                sendMessage(channel, request);
            }
        });

    }

}
