package cn.com.ty.chat.neety.process;

import cn.com.ty.chat.neety.protocol.IMMessage;
import cn.com.ty.chat.neety.protocol.IMP;
import cn.com.ty.chat.neety.support.WebSocketSupport;
import cn.com.ty.chat.neety.task.RoomInfo;
import com.alibaba.fastjson.JSONObject;
import io.netty.channel.Channel;
import lombok.extern.slf4j.Slf4j;

import java.util.Optional;

/**
 * 初始化房间操作类
 *
 * @author sKing
 * @Date 2020年6月12日
 */
@Slf4j
public class InitRoomProcessor extends WebSocketSupport implements ProcessorInterface {

    /**
     * websoket消息处理
     *
     * @param client
     * @param request
     */
    @Override
    public void process(Channel client, IMMessage request) {
        String currentUserId = client.attr(USERID).get();
        String currentUserHead = client.attr(HEAD_PIC).get();
        String currentUserName = client.attr(USERNAME).get();

        String SESSION_ID = request.getSessionid();
        String USER_ID = request.getUserId();

        //用户每次进来更新自己的信息
        //如果自己的信息不存在就把用户添加到聊天室中
        //如果聊天室存在，并且当前用户没有在聊天室中，把用户添加到聊天室中------start
        Optional.ofNullable(redis.get("ROOM:" + SESSION_ID)).ifPresent(s -> {
            RoomInfo roomInfo = JSONObject.parseObject(s,RoomInfo.class);
            roomInfo.getUserList().forEach(userInfo -> {
                if (userInfo.getUserId().equals(currentUserId)){
                    userInfo.setAvatarUrl(currentUserHead);
                    userInfo.setUserName(currentUserName);
                    userInfo.setUserId(currentUserId);
                    userInfo.setIsRed(0);
                }
            });


            redis.set("ROOM:" + SESSION_ID, JSONObject.toJSONString(roomInfo));
        });
        //如果聊天室存在，并且当前用户没有在聊天室中，把用户添加到聊天室中------end

        //用户每次进来把当天聊天记录通过后台全部推送过去
        Optional.ofNullable(redis.lRange("ALLCHATS:" + SESSION_ID,0,-1)).ifPresent(strings -> {
            IMMessage im = new IMMessage();
            im.setCmd(IMP.INITROOM.getName());
            im.setMsg(JSONObject.toJSONString(strings));
            im.setSessionid(SESSION_ID);
            sendMessage(client, im);
        });
    }

}
