package cn.com.ty.chat.neety.server;

import cn.com.ty.chat.neety.server.handler.WebSocketHandler;
import io.netty.channel.Channel;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.ssl.SslContext;
import io.netty.handler.ssl.SslHandler;
import io.netty.handler.stream.ChunkedWriteHandler;
import io.netty.handler.timeout.IdleStateHandler;

import javax.net.ssl.SSLEngine;
import java.util.concurrent.TimeUnit;

public class MyInitChannel extends ChannelInitializer {
    private SslContext context;
    private int readerIdleTime;

    public MyInitChannel(SslContext context, int readerIdleTime) {
        this.context = context;
        this.readerIdleTime = readerIdleTime;
    }

    @Override
    protected void initChannel(Channel channel) throws Exception {
        ChannelPipeline pipeline = channel.pipeline();
        if (context != null){
            SSLEngine sslEngine = context.newEngine(channel.alloc());
            sslEngine.setNeedClientAuth(false);
            sslEngine.setUseClientMode(false);
            pipeline.addLast("ssl", new SslHandler(sslEngine));
        }
        // 读、取、读取 心跳检测
        pipeline.addLast("ping", new IdleStateHandler(readerIdleTime, 0, 0, TimeUnit.SECONDS));
        // Http消息编码解码
        pipeline.addLast("http-codec", new HttpServerCodec());
        // 参数含义是消息合并的数据大小，如此代表聚合的消息内容长度不超过65536kb。
        pipeline.addLast("aggregator", new HttpObjectAggregator(65536));
        // WebSocket通信支持
        pipeline.addLast("http-chunked", new ChunkedWriteHandler());
        // http请求的业务逻辑处理 WebSocket服务端Handler
        pipeline.addLast("handler", new WebSocketHandler());
    }
}
